@@ -22,14 +22,10 @@
 from __future__ import annotations
 
 import abc
-import logging
 
 from pathlib import Path
 
-from molecule.exceptions import MoleculeError
-
-
-LOG = logging.getLogger(__name__)
+from molecule.util import sysexit_with_message
 
 
 class Base(abc.ABC):
@@ -46,4 +42,4 @@ class Base(abc.ABC):
     def _validate_template_dir(self, template_dir: str) -> None:
         if not Path(template_dir).is_dir():
             msg = f"The specified template directory ({template_dir!s}) does not exist"
-            raise MoleculeError(message=msg)
+            sysexit_with_message(msg, code=1)
