@@ -21,26 +21,26 @@
 
 from __future__ import annotations
 
-import logging
+import os
 import shlex
 import subprocess
 import warnings
 
+from pathlib import Path
 from typing import TYPE_CHECKING
 
-from molecule import util
+from rich.markup import escape
+
+from molecule import logger, util
 from molecule.api import MoleculeRuntimeWarning
 from molecule.exceptions import ScenarioFailureError
-from molecule.types import ScenarioResult
+from molecule.reporting.definitions import CompletionState
 
 
 if TYPE_CHECKING:
     from molecule.config import Config
 
 
-LOG = logging.getLogger(__name__)
-
-
 class AnsiblePlaybook:
     """Provisioner Playbook."""
 
@@ -72,6 +72,17 @@ class AnsiblePlaybook:
         elif self._config.provisioner:
             self._env = self._config.provisioner.env
 
+    @property
+    def _log(self) -> logger.ScenarioLoggerAdapter:
+        """Get a fresh scenario logger with current context.
+
+        Returns:
+            A scenario logger adapter with current scenario and step context.
+        """
+        # Get step context from the current action being executed
+        step_name = getattr(self._config, "action", "provisioner")
+        return logger.get_scenario_logger(__name__, self._config.scenario.name, step_name)
+
     def bake(self) -> None:
         """Bake ``ansible-playbook`` or ``navigator run`` command so it's ready to execute.
 
@@ -92,20 +103,8 @@ class AnsiblePlaybook:
                 if options.get("become"):
                     del options["become"]
 
-            # We do not pass user-specified Ansible arguments to the create and
-            # destroy invocations because playbooks involved in those two
-            # operations are not always provided by end users. And in those cases,
-            # custom Ansible arguments can break the creation and destruction
-            # processes.
-            #
-            # If users need to modify the creation of deletion, they can supply
-            # custom playbooks and specify them in the scenario configuration.
-            if self._config.action not in ["create", "destroy"]:
-                ansible_args = list(self._config.provisioner.ansible_args) + list(
-                    self._config.ansible_args,
-                )
-            else:
-                ansible_args = []
+            all_args = [*self._config.provisioner.ansible_args, *self._config.ansible_args]
+            ansible_args = all_args if self._should_provide_args(self._config.action) else []
 
             backend = self._config.executor
 
@@ -117,7 +116,7 @@ class AnsiblePlaybook:
                         text=True,
                         check=True,
                     )
-                    LOG.info("%s version: %s", backend, result.stdout.strip())
+                    self._log.debug("%s version: %s", backend, result.stdout.strip())
                 except subprocess.CalledProcessError as exc:
                     msg = f"{backend} is not available. Please ensure that it is installed."
                     raise RuntimeError(msg) from exc
@@ -162,9 +161,10 @@ class AnsiblePlaybook:
             self.bake()
 
         if not self._playbook:
-            LOG.warning("Skipping, %s action has no playbook.", self._config.action)
-            self._config.scenario.results.append(
-                ScenarioResult(subcommand=self._config.action, state="SKIPPED"),
+            message = "Missing playbook"
+            note = f"Remove from {self._config.subcommand}_sequence to suppress"
+            self._config.scenario.results.add_completion(
+                CompletionState.missing(message=message, note=note),
             )
             return ""
 
@@ -177,28 +177,23 @@ class AnsiblePlaybook:
                 env=self._env,
                 debug=self._config.debug,
                 cwd=cwd,
+                command_borders=self._config.command_borders,
             )
 
         if result.returncode != 0:
-            self._config.scenario.results.append(
-                ScenarioResult(subcommand=self._config.action, state="FAILED"),
-            )
+            err = f"Ansible return code was {result.returncode}, command was: {escape(shlex.join(result.args))}"
+            self._config.scenario.results.add_completion(CompletionState.failed(note=err))
 
-            from rich.markup import escape
-
-            msg = f"Ansible return code was {result.returncode}, command was: [dim]{escape(shlex.join(result.args))}[/dim]"
             raise ScenarioFailureError(
-                msg,
+                err,
                 code=result.returncode,
                 warns=warns,
             )
 
-        self._config.scenario.results.append(
-            ScenarioResult(subcommand=self._config.action, state="PASSED"),
-        )
+        self._config.scenario.results.add_completion(CompletionState.successful)
         return result.stdout
 
-    def add_cli_arg(self, name: str, value: str | bool) -> None:
+    def add_cli_arg(self, name: str, value: str | bool) -> None:  # noqa: FBT001
         """Add argument to CLI passed to ansible-playbook.
 
         Args:
@@ -216,3 +211,28 @@ class AnsiblePlaybook:
             value: The value of argument to be added.
         """
         self._env[name] = value
+
+    def _should_provide_args(self, action: str | None) -> bool:
+        """Check if ansible_args should be provided for this action.
+
+        Args:
+            action: The action name (create, destroy, etc.)
+
+        Returns:
+            True if ansible_args should be provided, False otherwise.
+        """
+        # Strict mode: be conservative, exclude create/destroy and None actions
+        if os.getenv("MOLECULE_ANSIBLE_ARGS_STRICT_MODE"):
+            return action not in ["create", "destroy", None]
+
+        # No provisioner: fall back to original behavior (None gets args)
+        if not self._config.provisioner:
+            return action not in ["create", "destroy"] if action else True
+
+        # Smart mode: exclude create/destroy only for bundled playbooks
+        if action in ["create", "destroy"]:
+            playbook = getattr(self._config.provisioner.playbooks, action, None)
+            return bool(playbook and Path(playbook).exists())
+
+        # Default: provide args (None actions, non-create/destroy actions)
+        return True
