@@ -2,12 +2,12 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, TypedDict
+from typing import TYPE_CHECKING, Any, Literal, TypedDict
 
 
 if TYPE_CHECKING:
     from collections.abc import MutableMapping
-    from typing import Any, Literal, TypeAlias
+    from typing import TypeAlias
 
     Options: TypeAlias = MutableMapping[str, str | bool]
 
@@ -190,6 +190,46 @@ class ScenarioData(TypedDict):
     test_sequence: list[str]
 
 
+class ExecutorArgsData(TypedDict, total=False):
+    """Executor-specific arguments.
+
+    Attributes:
+        ansible_navigator: Arguments passed to ansible-navigator.
+        ansible_playbook: Arguments passed to ansible-playbook.
+    """
+
+    ansible_navigator: list[str]
+    ansible_playbook: list[str]
+
+
+class ExecutorData(TypedDict, total=False):
+    """Executor configuration.
+
+    Attributes:
+        backend: The executor backend to use (ansible-playbook or ansible-navigator).
+        args: Executor-specific arguments.
+    """
+
+    backend: str
+    args: ExecutorArgsData
+
+
+class AnsibleData(TypedDict, total=False):
+    """Ansible-specific configuration section.
+
+    Attributes:
+        cfg: Ansible configuration options (maps to ansible.cfg).
+        executor: Executor configuration including backend and arguments.
+        env: Environment variables for ansible execution.
+        playbooks: Playbook paths for different scenarios.
+    """
+
+    cfg: dict[str, Any]
+    executor: ExecutorData
+    env: dict[str, str]
+    playbooks: PlaybookData
+
+
 class VerifierData(TypedDict, total=False):
     """Molecule verifier configuration.
 
@@ -210,20 +250,26 @@ class VerifierData(TypedDict, total=False):
     additional_files_or_dirs: list[str]
 
 
-class ConfigData(TypedDict):
-    """Class representing molecule config.
+class DefaultConfigData(TypedDict):
+    """Default configuration structure with forward-looking ansible section.
+
+    This represents the complete default configuration including all sections
+    with their proper types and structure. Used for DEFAULT_CONFIG constant.
 
     Attributes:
-        dependency: Dependency config.
-        driver: Driver config.
-        platforms: List of platforms.
-        prerun: Should prerun tasks be run.
-        role_name_check: ???
-        provisioner: Provisioner config.
-        scenario: Scenario config.
-        verifier: Verifier config.
+        ansible: Ansible-specific configuration section.
+        dependency: Dependency management configuration.
+        driver: Driver configuration for platform management.
+        platforms: List of target platforms.
+        prerun: Whether to run prerun tasks.
+        role_name_check: Role name validation level.
+        provisioner: Provisioner configuration (legacy, minus migrated keys).
+        scenario: Scenario execution configuration.
+        shared_state: Whether to share state between scenarios.
+        verifier: Verifier configuration for testing.
     """
 
+    ansible: AnsibleData
     dependency: DependencyData
     driver: DriverData
     platforms: list[PlatformData]
@@ -231,6 +277,35 @@ class ConfigData(TypedDict):
     role_name_check: int
     provisioner: ProvisionerData
     scenario: ScenarioData
+    shared_state: bool
+    verifier: VerifierData
+
+
+class ConfigData(TypedDict, total=False):
+    """Class representing molecule config.
+
+    Attributes:
+        ansible: Ansible-specific configuration (new format).
+        dependency: Dependency config.
+        driver: Driver config.
+        platforms: List of platforms.
+        prerun: Should prerun tasks be run.
+        role_name_check: Role name validation level.
+        provisioner: Provisioner config.
+        scenario: Scenario config.
+        shared_state: Should state be shared between scenarios.
+        verifier: Verifier config.
+    """
+
+    ansible: AnsibleData
+    dependency: DependencyData
+    driver: DriverData
+    platforms: list[PlatformData]
+    prerun: bool
+    role_name_check: int
+    provisioner: ProvisionerData
+    scenario: ScenarioData
+    shared_state: bool
     verifier: VerifierData
 
 
@@ -265,7 +340,9 @@ class CommandArgs(TypedDict, total=False):
         platform_name: Name of the platform to target.
         report: Whether to show an after-run summary report.
         scenario_name: Name of the scenario to target.
+        shared_state: Whether (some) state should be shared between scenarios.
         subcommand: Name of subcommand being run.
+        command_borders: Whether to enable borders around command output.
     """
 
     destroy: Literal["always", "never"]
@@ -277,28 +354,6 @@ class CommandArgs(TypedDict, total=False):
     platform_name: str
     report: bool
     scenario_name: str
+    shared_state: bool
     subcommand: str
-
-
-class ScenarioResult(TypedDict):
-    """Dictionary containing the result of a Scenario action.
-
-    Attributes:
-        subcommand: The action that ran.
-        state: The outcome of the action ("PASSED", "FAILED", or "SKIPPED).
-    """
-
-    subcommand: str | None
-    state: Literal["PASSED", "FAILED", "SKIPPED"]
-
-
-class ScenariosResults(TypedDict):
-    """Dictionary containing the results of multiple Scenario runs.
-
-    Attributes:
-        name: The name of the scenario.
-        results: A list of ScenarioResult objects.
-    """
-
-    name: str
-    results: list[ScenarioResult]
+    command_borders: bool
